package com.atlassian.elasticsearch.buckler;

import com.atlassian.elasticsearch.buckler.config.BucklerConfig;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.common.util.concurrent.ThreadContext;
import org.elasticsearch.transport.*;

class AuthorisedSender implements TransportInterceptor.AsyncSender {

    private static final Logger log = LogManager.getLogger(AuthorisedSender.class.getName());

    private final TransportInterceptor.AsyncSender sender;
    private final BucklerConfig bucklerConfig;
    private final ThreadContext threadContext;

    public AuthorisedSender(TransportInterceptor.AsyncSender sender, BucklerConfig bucklerConfig, ThreadContext threadContext) {
        this.sender = sender;
        this.bucklerConfig = bucklerConfig;
        this.threadContext = threadContext;
    }

    @Override
    public <T extends TransportResponse> void sendRequest(Transport.Connection connection, String action, TransportRequest request, TransportRequestOptions options, TransportResponseHandler<T> handler) {
        if (bucklerConfig.getAuthConfig().isEnabledForTcp()) {
            String authorizationConfigured = bucklerConfig.getAuthConfig().getAuthorization();
            String authorizationRequest = threadContext.getHeader("Authorization");
            if (authorizationRequest == null) {
                log.trace("Request {} missing Authorization header, adding {}", action, authorizationConfigured);
                threadContext.putHeader("Authorization", authorizationConfigured);
            } else {
                log.trace("Request {} already has Authorization header, using {}", action, authorizationRequest);
            }
        }

        sender.sendRequest(connection, action, request, options, handler);
    }
}
