package com.atlassian.elasticsearch.buckler.config;

import com.atlassian.elasticsearch.buckler.BasicAuth;
import org.elasticsearch.common.settings.Settings;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.util.Arrays;

public class AuthConfig {

    private final boolean enabledForHttp;
    private final boolean enabledForTcp;
    private final byte[] authorization;

    AuthConfig(Settings settings) {
        this.enabledForHttp = BucklerConfig.loadEnabled(settings, "auth.basic.http.enabled", "Basic authentication for HTTP transport");
        this.enabledForTcp = BucklerConfig.loadEnabled(settings, "auth.basic.tcp.enabled", "Basic authentication for TCP transport");

        if (enabledForHttp || enabledForTcp) {
            String username = BucklerConfig.getRequiredSetting(settings, "auth.basic.username",
                    "Username must be configured for basic authentication");
            String password = BucklerConfig.getRequiredSetting(settings, "auth.basic.password",
                    "Password must be configured for basic authentication");
            this.authorization = BasicAuth.encodeCredentials(username, password).getBytes(StandardCharsets.UTF_8);
        } else {
            this.authorization = new byte[]{};
        }
    }

    public boolean isEnabledForHttp() {
        return enabledForHttp;
    }

    public boolean isEnabledForTcp() {
        return enabledForTcp;
    }

    public boolean isAuthorized(String authorizationValue) {
        return MessageDigest.isEqual(authorization, authorizationValue.getBytes(StandardCharsets.UTF_8));
    }

    public String getAuthorization() {
        return new String(authorization, StandardCharsets.UTF_8);
    }
}
